{
  fpYate - Yate external module implementation for Free Pascal
  (c) 2010 Ionut Muntean ionut@muntean.ro
  version 0.1

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
}

unit fpYate;

{$mode objfpc}{$H+}

interface

//{$define doLnet}

uses
  Classes, SysUtils,

{$ifdef doLnet}
  lNet,
{$endif}

  StrUtils, BaseUnix, DateUtils;


type

    TExplodeArray = array of String;

    { TYate }
    TYate = class
      yId: String;
      yType: String;
      yName: String;
      yRetVal: String;
      yOrigin: Longint;
      yHandled: Boolean;
      yParams: TStringList;
      function YateEscape(Str: String; Extra: Char = #0): String;
      function YateUnEscape(Str: String): String;
      function YateInstall(Name: String; Priority: Integer = 100; FiltName: String = ''; FiltValue: String = ''): Boolean;
      function YateUninstall(Name: String): Boolean;
      function YateWatch(Name: String): Boolean;
      function YateUnWatch(Name: String): Boolean;
      function YateSetLocal(Name, Value: String): Boolean;
      procedure YateSetParam(Key, Value: String);
      function YateGetValue(Key: String; DefaultValue: String = ''): String;
      procedure YateFillParams(Values: TExplodeArray; Offset: Integer = 0);
      function YateDispatch: Boolean;
      function YateAcknoledge: Boolean;
      function YateGetEvent: TYate;
      constructor Create(Name: String = ''; Retval: String = ''; Id: String = ''); overload;
      destructor Destroy; override;
    end;

function Implode(const cSeparator: String; const cArray: TExplodeArray): String;
function Explode(const cSeparator, vString: String): TExplodeArray;

function YateWrite(Str: String): Integer;
function YateInit(Addr: String = ''; Port: Integer = 0; Role: String = ''): Boolean;
function YateOutput(Str: String): Boolean;
function YateDebug(Str: String): Boolean;

{

 You need lNet package from http://lnet.wordpress.com/ if you want to use
 TCP connection to Yate.

}

{$ifdef doLNet}
var
    YateClient: TLTcp;
{$endif}

implementation

const
    yDebug: Boolean = true;
    Buffer: String = '';
    isTcp: Boolean = false;
    yIn: Longint = 0;
    yOut: LongInt = 1;
    yErr: LongInt = 2;


// Helper functions
function Implode(const cSeparator: String; const cArray: TExplodeArray): String;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to Length(cArray) -1 do begin
    Result := Result + cSeparator + cArray[i];
  end;
  System.Delete(Result, 1, Length(cSeparator));
end;

function Explode(const cSeparator, vString: String): TExplodeArray;
var
  i: Integer;
  S: String;
begin
  S := vString;
  SetLength(Result, 0);
  i := 0;
  while Pos(cSeparator, S) > 0 do begin
    SetLength(Result, Length(Result) +1);
    Result[i] := Copy(S, 1, Pos(cSeparator, S) -1);
    Inc(i);
    S := Copy(S, Pos(cSeparator, S) + Length(cSeparator), Length(S));
  end;
  SetLength(Result, Length(Result) +1);
  Result[i] := Copy(S, 1, Length(S));
end;


function YateWrite(Str: String): Integer;
begin
  Str := Str + #10;
{$ifdef doLNet}
  if isTcp and Assigned(YateClient) then
      Result := YateClient.SendMessage(Str)
    else
{$endif}
  Result := FpWrite(yOut, Str[1], Length(Str));
end;

function YateOutput(Str: String): Boolean;
begin
  Result := false;
  YateWrite('%%>output:' + Str);
end;

function YateDebug(Str: String): Boolean;
begin
  Result := false;
  if not yDebug then Exit;
  YateOutput(Str);
end;

function YateInit(Addr: String = ''; Port: Integer = 0; Role: String = ''): Boolean;
var Cnt: Longint;
begin
  if Addr <> '' then
  begin
    isTcp := true;
    YateDebug('Yate.YateInit -> Init. Connecting to Yate server at ' + Addr + ':' + IntToStr(Port));
    Result := false;
{$ifdef doLNet}
    YateClient := TLTcp.Create(nil);
    YateClient.Timeout := 100;
    Buffer := '';
    Cnt := 0;
    try
      if YateClient.Connect(Addr, Port) then
       repeat
          YateClient.CallAction;
          if TimeOut(Cnt, GwTimeOut) then
             begin
               YateDebug('Yate.YateInit -> Timeout waiting for Yate`s response. [' + IntToStr(GwTimeOut) + ' sec]');
               Exit;
             end;
       until YateClient.Connected;
    except
      on E: Exception do
      begin
        YateDebug(Format('Yate.YateInit -> Yate connection failure for %s on port %d, Error: %s.', [Addr, Port, E.Message]));
        Exit;
      end;
    end;
    Result := true;
{$endif}
  end
  else
   begin
     isTcp := false;
     Result := true;
   end;
  if Role <> '' then
     YateWrite('%%>connect:' + Role);
end;

function TYate.YateEscape(Str: String; Extra: Char): String;
var i: Integer;
    S: String;
    C: Char;
begin
  S := '';
  for i := 1 to Length(Str) do
    begin
      C := Str[i];
      if (Ord(C) < 32) or (C = ':') or (C = Extra) then
       begin
         C := Chr(Ord(C) + 64);
         S := S + '%';
       end
      else if (C = '%') then
        S := S + C;
      S := S + C;
    end;
  Result := S;
end;

function TYate.YateUnEscape(Str: String): String;
var i: Integer;
    S: String;
    C: Char;
begin
  S := '';
  i := 1;
  while i <= Length(Str) do
  begin
    C := Str[i];
    if C = '%' then
     begin
       Inc(i);
       C := Str[i];
       if C <> '%' then
         C := Chr(Ord(C) - 64);
     end;
    S := S + C;
    Inc(i);
  end;
  Result := S;
end;

function TYate.YateInstall(Name: String; Priority: Integer; FiltName: String; FiltValue: String): Boolean;
var Str: String;
begin
  Result := false;
  if FiltName <> '' then
    FiltName := ':' + FiltName + ':' + FiltValue;
  Str := '%%>install:' + IntToStr(Priority) + ':' + YateEscape(Name) + FiltName;
  YateWrite(Str);
end;

function TYate.YateUninstall(Name: String): Boolean;
var Str: String;
begin
  Result := false;
  Str := '%%>uninstall: ' + YateEscape(Name);
  YateWrite(Str);
end;

function TYate.YateWatch(Name: String): Boolean;
var Str: String;
begin
  Result := false;
  Str := '%%>watch: ' + YateEscape(Name);
  YateWrite(Str);
end;

function TYate.YateUnWatch(Name: String): Boolean;
var Str: String;
begin
  Result := false;
  Str := '%%>unwatch: ' + YateEscape(Name);
  YateWrite(Str);
end;

function TYate.YateSetLocal(Name, Value: String): Boolean;
var Str: String;
begin
  Result := false;
  Str := '%%>setlocal:' + YateEscape(Name) + ':' + YateEscape(Value);
  YateWrite(Str);
end;

procedure TYate.YateSetParam(Key, Value: String);
begin
  yParams.Values[Key] := Value;
end;

function TYate.YateGetValue(Key: String; DefaultValue: String): String;
begin
  if yParams.Values[Key] = '' then Result := DefaultValue else Result := yParams.Values[Key];
end;

procedure TYate.YateFillParams(Values: TExplodeArray; Offset: Integer = 0);
var i: Integer;
    W: TExplodeArray;
begin
  for i := Offset to Length(Values) - 1 do
   begin
     W := Explode('=', Values[i]);
     yParams.Values[YateUnescape(W[0])] := YateUnEscape(W[1]);
   end;
  SetLength(W, 0);
end;

function TYate.YateDispatch: Boolean;
var Str: String;
    i: Integer;
begin
  Result := false;
  if yType <> 'outgoing' then
   begin
     YateDebug('Yate.YateDispatch -> Attempt to dispatch a message of type ' + yType);
     Exit;
   end;
  Str := '%%>message:' + YateEscape(yId) + ':' + IntToStr(yOrigin) + ':' + YateEscape(yName) + ':' + YateEscape(yRetVal);
  for i := 0 to yParams.Count - 1 do
   Str := Str + ':' + yParams.Names[i] + '=' + yParams.ValueFromIndex[i];
  yType := 'dispatched';
  YateWrite(Str);
end;

function TYate.YateAcknoledge: Boolean;
var Str: String;
    i: Integer;
begin
  Result := false;
  if yType <> 'incoming' then
   begin
     YateDebug('Yate.YateAcknoledge -> Attempt to acknoledge a message of type ' + yType);
     Exit;
   end;
  Str := '%%<message:' + YateEscape(yId) + ':' + BoolToStr(yHandled, 'true', 'false') + ':' + YateEscape(yName) + ':' + YateEscape(yRetVal);
  for i := 0 to yParams.Count - 1 do
   Str := Str + ':' + yParams.Names[i] + '=' + yParams.ValueFromIndex[i];
  yType := 'acknoledged';
  YateWrite(Str);
end;

function TYate.YateGetEvent: TYate;
var Line: String;
    Part: TExplodeArray;
    W: TExplodeArray;
    i: Integer;
    C: Char;
begin
  Result := nil;
  Line := '';
  if isTcp then
   begin
{$ifdef doLNet}
      while true do
       begin
        YateClient.CallAction;
        if YateClient.GetMessage(Line) > 0 then
         begin
           Buffer := Buffer + Line;
           if Pos(#10, Buffer) > 0 then
            begin
              Line := Copy(Buffer, 1, Pos(#10, Buffer));
              Delete(Buffer, 1, Pos(#10, Buffer));
              Break;
            end;
         end;
       end;
{$endif}
   end
  else
   begin
     Line := '';
     while true do
       begin
         if FpRead(yIn, C, 1) = 1 then
         if C <> #10 then
           Line := Line + C
          else
           Break
       end;
   end;
  if Pos(#10, Line) > 0 then
    Line := AnsiReplaceStr(Line, #10, '');
  if Line = '' then Exit;
  try
    Part := Explode(':', Line);
    if Part[0] = '%%>message' then
     begin
       Result := TYate.Create(YateUnEscape(Part[3]), YateUnEscape(Part[4]), YateUnEscape(Part[1]));
       Result.yType := 'incoming';
       Result.yOrigin := StrToInt(Part[2]);
       Result.YateFillParams(Part, 5);
     end else
    if Part[0] = '%%<message' then
     begin
       Result := TYate.Create(YateUnEscape(Part[3]), YateUnEscape(Part[4]), YateUnEscape(Part[1]));
       Result.yType := 'answer';
       Result.yHandled := StrToBool(Part[2]);
       Result.YateFillParams(Part, 5);
     end else
    if Part[0] = '%%<install' then
     begin
       Result := TYate.Create(YateUnEscape(Part[2]), '', YateUnEscape(Part[1]));
       Result.yType := 'installed';
       Result.yHandled := StrToBool(Part[3]);
     end else
    if Part[0] = '%%<uninstall' then
     begin
       Result := TYate.Create(YateUnEscape(Part[2]), '', YateUnEscape(Part[1]));
       Result.yType := 'uninstalled';
       Result.yHandled := StrToBool(Part[3]);
     end else
    if Part[0] = '%%<watch' then
     begin
       Result := TYate.Create(YateUnEscape(Part[1]), '', '');
       Result.yType := 'watched';
       Result.yHandled := StrToBool(Part[2]);
     end else
    if Part[0] = '%%<unwatch' then
     begin
       Result := TYate.Create(YateUnEscape(Part[1]), '', '');
       Result.yType := 'unwatched';
       Result.yHandled := StrToBool(Part[2]);
     end else
    if Part[0] = '%%<connect' then
     begin
       Result := TYate.Create(YateUnEscape(Part[1]), '', '');
       Result.yType := 'connected';
       Result.yHandled := StrToBool(Part[2]);
     end else
    if Part[0] = '%%<setlocal' then
     begin
       Result := TYate.Create(YateUnEscape(Part[1]), YateUnEscape(Part[2]), '');
       Result.yType := 'setlocal';
       Result.yHandled := StrToBool(Part[3]);
     end else
    if Part[0] = 'Error in' then // Error?
        Exit
     else
        YateDebug('Yate.YateGetEvent -> Parse Error on line: ' + Line);
  finally
    SetLength(Part, 0);
  end;
end;

constructor TYate.Create(Name: String; Retval: String = ''; Id: String = '');
var Cnt: Longint;
begin
  if Id = '' then
    yId := 'fpyate/' + IntToStr(DateTimeToUnix(Now))
   else
    yId := Id;
  yName := Name;
  yRetVal := Retval;
  yType := 'outgoing';
  yOrigin := DateTimeToUnix(Now);
  yHandled := false;
  yParams := TStringList.Create;
end;

destructor TYate.Destroy;
begin
  if Assigned(yParams) then yParams.Free;
  inherited Destroy;
end;


finalization
{$ifdef doLNet}
  if isTcp and Assigned(YateClient) then YateClient.Free;
{$endif}

end.

