{
  fpYate - Yate external module implementation for Free Pascal
  (c) 2010 Ionut Muntean ionut@muntean.ro
  version 0.1

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
}

{
   Command line dialer. Put fpyaterecorder and fpyatedialerin the scripts
   directory defined in extmodule.conf. Edit extmodule.conf and add this line
   in the [scripts] section:

   fpyatedialer=xxxxxxxx,recording.au

   where xxxxxxxx is the number to dial and recording.au is the path and filename
   of the recording.

   Restart Yate.

   Yate should be able to dial the number and record the other party's voice
   to file recording.au ...

}

program fpYateDialer;

{$mode objfpc}{$H+}

uses
  Classes, SysUtils, fpYate;

const YateRecorder: String = 'external/nodata/fpyaterecorder';
      Run: Boolean = true;
      RecordFile: String = 'test.au';
      CallerId: String = '89898989'; // change this to whatever callerid you want.
      OurCallId: String = '';
      wasRouted: Boolean = false;

procedure startRouting(Number: String);
var Y: TYate;
begin
  YateDebug('YateDial.startRouting -> Routing call for number "' + Number + '"');
  Y := TYate.Create('call.route');
  Y.YateSetParam('caller', CallerId);
  Y.YateSetParam('callername', 'fpyate');
  Y.YateSetParam('called', Number);
  Y.YateDispatch;
  Y.Free;
  wasRouted := true;
end;

procedure callInitate(Target: String; Ev: TYate);
var Y: TYate;
begin
  YateDebug('YateDial.callInitate -> Initating call to target "' + Target + '"');
  Y := TYate.Create('call.execute');
  Y.yId := '';
  Y.YateSetParam('callto', YateRecorder + ' ' + RecordFile); // pass record file name to fpyaterecorder as a single parameter
  Y.YateSetParam('direct', Target);
  Y.YateSetParam('caller', Ev.YateGetValue('caller'));
  Y.YateSetParam('callername', Ev.YateGetValue('callername'));
//  Y.YateSetParam('format', 'g729');                        // does this work??
  Y.YateDispatch();
  Y.Free;
  Run := false;
end;

procedure routeFailure(Error: String; Ev: TYate);
begin
  YateDebug('YateDial.routeFailure -> Failed to route number ' + Ev.YateGetValue('called') + ' Error: ' + Error);
  Halt;
end;

procedure YateDialer(Number: String; RecordFileName: String = '');
var Y: TYate;
    Ev: TYate;
begin
  if RecordFileName <> '' then
     RecordFile := RecordFileName;
  Y := TYate.Create;
  Y.YateInstall('engine.timer');
  Y.YateSetLocal('restart', 'false'); // we want only one run ...
  while Run do
  begin
    Ev := Y.YateGetEvent;
    if Ev = nil then Continue;
    if Ev.yType = 'incoming' then
     begin
       Ev.YateAcknoledge;
       startRouting(Number);
     end else
     if Ev.yType = 'answer' then
     begin
       if Ev.yName = 'call.route' then
         begin
           if Ev.yHandled and (Ev.yRetVal <> '') and (Ev.yRetVal <> '-') and (Ev.yRetVal <> 'error') then
             callInitate(Ev.yRetVal, Ev)
            else
             routeFailure(Ev.YateGetValue('error'), Ev);
         end;
     end;
     Ev.Free;
  end;
  Y.Free;
end;

var W: TExplodeArray;

begin
  W := Explode(',', ParamStr(1));
  YateInit();
  YateDialer(W[0], W[1]);
  SetLength(W, 0);
end.

