{
  fpYate - Yate external module implementation for Free Pascal
  (c) 2010 Ionut Muntean ionut@muntean.ro
  version 0.1

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
}

{
   Command line dialer. Put fpyaterecorder and fpyatedialerin the scripts
   directory defined in extmodule.conf. Edit extmodule.conf and add this line
   in the [scripts] section:

   fpyatedialer=xxxxxxxx,recording.au

   where xxxxxxxx is the number to dial and recording.au is the path and filename
   of the recording.

   Restart Yate.

   Yate should be able to dial the number and record the other party's voice
   to file recording.au ...

}

program fpyaterecorder;

{$mode objfpc}{$H+}

uses
  Classes, SysUtils, fpYate, DateUtils;

const Path: String = '';
      Prefix: String = '';
      Number: String = '';
      CallerId: String = '';
      Run: Boolean = true;
      Done: Boolean = false;
      OurCallId: String = '';
      PartyCallid: String = '';
      Caller: String = '';
      CallerName: String = '';
      Called: String = '';
      Dtmf: String = '';

procedure callEnd(Error: String; Reason: String = '');
begin
  if Done then Exit;
  if Error = '' then Error := Reason;
  if Error = 'Normal Clearing' then Error := 'NORMAL_CLEARING';
  YateDebug('YateRecord.callEnd -> Call ended with result: ' + Error);
  Run := false;
  Done := true;
end;

procedure callExecuting(PeerId: String);
var Y: TYate;
begin
  YateDebug('YateRecord.callExecuting -> Executing call to ' + Number + ', Peerid is ' + PeerId);
  PartyCallid := PeerId;
  Y := TYate.Create();
  Y.YateInstall('chan.dtmf', 90, 'peerid', OurCallId);
  Y.YateInstall('call.answered', 90, 'peerid', OurCallId);
  Y.YateInstall('chan.notify', 90, 'id', OurCallId);
  Y.Free;
end;

procedure callAnswered;
var Y: TYate;
begin
  YateDebug('YateRecord.callAnswered -> Call to ' + Number + ' answered. Recording to ' + ParamStr(1));
  Y := TYate.Create('chan.attach');
  Y.yParams.Values['source'] := 'wave/play/-';
  Y.yParams.Values['consumer'] := 'wave/record/' + ParamStr(1);
  Y.yParams.Values['notify'] := OurCallId;
  Y.YateDispatch;
  Y.Free;
end;

procedure YateRecorder;
var Ev, Y, Ex: TYate;
    S: String;
    W: TExplodeArray;
begin
  YateDebug('YateRecord.YateRecorder -> Starting up ...');
  Y := TYate.Create;
//  Y.YateInstall('engine.timer');
  OurCallId := 'fpyaterecorder/' + IntToStr(DateTimeToUnix(Now));
  Y.YateSetLocal('id', OurCallId);
  Y.YateSetLocal('disconnected', 'true');
  Y.YateInstall('chan.disconnected', 90, 'id', OurCallId);

  while Run do
  begin
    Ev := Y.YateGetEvent;
    if Ev = nil then Continue;
    if Ev.yType = 'incoming' then
     begin
       if Ev.yName = 'engine.timer' then
        begin
         //  do something on timer
        end else
       if Ev.yName = 'call.execute' then
        begin
          Caller := Ev.YateGetValue('caller');
          CallerName := Ev.YateGetValue('callername');
          Called := Ev.YateGetValue('called');
          Ev.yHandled := true;
          Ex := TYate.Create('call.execute');
          Ex.YateSetParam('id', OurCallId);
          Ex.YateSetParam('callto', Ev.YateGetValue('direct'));
          Ex.YateSetParam('caller', Caller);
          Ex.YateSetParam('callername', CallerName);
          Ex.YateSetParam('called', Called);
          Ex.YateSetParam('cdrtrack', 'false');
          Ex.YateSetParam('tonedetect_out', 'true');
          Ex.YateSetParam('tonedetect_in', 'true');
          Ex.YateSetParam('format', Ev.YateGetValue('format'));
          Ex.YateDispatch;
          Ex.Free
        end else
       if Ev.yName = 'call.answered' then
        begin
          callAnswered;
          Ev.yHandled := true;
        end else
       if Ev.yName = 'chan.notify' then
        begin
          callEnd(Ev.YateGetValue('reason'));
          YateDebug('YateRecord.YateRecorder -> Chan Notify: ' + Ev.YateGetValue('reason'));
          Ev.yHandled := true
        end else
       if Ev.yName = 'chan.dtmf' then
        begin
          Dtmf := Dtmf + Ev.YateGetValue('text');
          Ev.yHandled := true;
        end else
       if Ev.yName = 'chan.disconnected' then
        begin
          callEnd(Ev.YateGetValue('reason'));
//          Run := true;
          Ev.yHandled := true
        end;
       Ev.YateAcknoledge;
     end else
    if Ev.yType = 'answer' then
     begin
       if Ev.yName = 'call.execute' then
        begin
          if Ev.yHandled then
             callExecuting(Ev.YateGetValue('peerid'))
           else
             callEnd(Ev.YateGetValue('error'), Ev.YateGetValue('reason'));
        end
     end;
    Ev.Free;
  end;
  YateDebug('YateRecord.YateRecorder -> Exit run. DTMF = "' + Dtmf + '".');
  Y.Free;
end;

begin
  YateInit();
  YateRecorder;
end.

